<?php

namespace Ems\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;


use Ems\Services\ReportService;

class ReportController {
    /** @var ReportService */
    protected $reportService;

    public function __construct(
        ReportService $reportService
    ) {
        $this->reportService = $reportService;
    }

    public function generateBirtReport(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();

            $link = $this->reportService->generateBirtReport($row);

            $dataset = [
                'link' => $link
            ];

            $response->addDataSet(new DataSet('generateBirtReport', $dataset));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function downloadReport(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();
            $link = (string) $row['link'];

            $file = $this->reportService->downloadReport($link);

            $response->addDataSet(new DataSet('downloadReport', $file));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}
