<?php

namespace Ems\Services;

use Ems\Repository\BancoHorasRepository;
use Util\Environment;
use Util\NovoCodigo;

class BancoHorasService {
    /** @var Environment */
    protected $environment;

    /** @var BancoHorasRepository */
    protected $bancoHorasRepository;

    public function __construct(
        Environment             $environment,
        BancoHorasRepository    $bancoHorasRepository
    ) {
        $this->environment          = $environment;
        $this->bancoHorasRepository = $bancoHorasRepository;
    }

    public function getBancoHorasAdmi(
        int $nrTipoVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $bancoHorasAdmi = $this->bancoHorasRepository->getBancoHorasAdmi(
            $nrOrg, $nrTipoVinculo
        );

        return $bancoHorasAdmi;
    }

    public function setAlteBancoHoras(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $params = [];

        $params['NRALTEBANCOHORAS'] =
            $row['NRALTEBANCOHORAS'] ??
            NovoCodigo::geraCodigo($nrOrg, 'PTO_ALTEBANCOHORAS', 1, 12, true, false);

        $params['NRORG']                = $row['NRORG'] ?? $nrOrg;
        $params['NRBANCOHORAS']         = (int) $row['NRBANCOHORAS'];
        $params['NRVINCULOM']           = (int) $row['NRVINCULOM'];
        $params['DTINICIOALTEBHORAS']   = (string) $row['DTINICIOALTEBHORAS'];
        $params['DTFIMALTEBHORAS']      = $row['DTFIMALTEBHORAS'] ?? null;
        $params['IDATIVO']              = $row['IDATIVO'] ?? 'S';
        $params['NRORGINCLUSAO']        = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $params['DTINCLUSAO']           = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $params['CDOPERINCLUSAO']       = $row['CDOPERINCLUSAO'] ?? 1;
        $params['NRORGULTATU']          = $row['NRORGULTATU'] ?? null;
        $params['DTULTATU']             = $row['DTULTATU'] ?? null;
        $params['CDOPERULTATU']         = $row['CDOPERULTATU'] ?? null;

        $this->bancoHorasRepository->setAlteBancoHoras($params);

        return $params['NRALTEBANCOHORAS'];
    }
}